#include <stdio.h>
#include <stdlib.h>
#include "stdlib.h"
#include <string.h>
#include <math.h>
#include "defines.h"
#include "ga_dyads.h"

void print_motif(Sites *site,int nsites,char **seq,char **rseq,int *seqLen,int pwmLen,int id) {

   FILE *fp;
   char *fileName;
   register int i,j;

   fileName=alloc_char(500);
   sprintf(fileName,"%d",id);
   fp=fopen(fileName,"w");
   for (i=0; i<nsites; i++) {
      if (site[i].rev=='0') {
         if (site[i].pos<0) {
            for (j=site[i].pos; j<0; j++) fprintf(fp,"x"); 
            for (j=0; j<pwmLen+site[i].pos; j++) {
               switch(seq[site[i].seq][j]) {
                  case 'a': fprintf(fp,"a"); break;
                  case 'c': fprintf(fp,"c"); break;
                  case 'g': fprintf(fp,"g"); break;
                  case 't': fprintf(fp,"t"); break;
                  case 'n': fprintf(fp,"n"); break;
                  default: break;
               }
            }
         }
         else {
            for (j=site[i].pos; j<min(seqLen[site[i].seq],site[i].pos+pwmLen); j++) {
               switch(seq[site[i].seq][j]) {
                  case 'a': fprintf(fp,"a"); break;
                  case 'c': fprintf(fp,"c"); break;
                  case 'g': fprintf(fp,"g"); break;
                  case 't': fprintf(fp,"t"); break;
                  case 'n': fprintf(fp,"n"); break;
                  default: break;
               }
            }
         }
         if (site[i].pos+pwmLen-seqLen[site[i].seq]>0) {
            for (j=seqLen[site[i].seq]; j<site[i].pos+pwmLen; j++) fprintf(fp,"x"); 
         }
         fprintf(fp,"\n");
      }
      else {
         if (site[i].pos<0) {
            for (j=site[i].pos; j<0; j++) fprintf(fp,"x"); 
            for (j=0; j<pwmLen+site[i].pos; j++) {
               switch(rseq[site[i].seq][j]) {
                  case 'a': fprintf(fp,"a"); break;
                  case 'c': fprintf(fp,"c"); break;
                  case 'g': fprintf(fp,"g"); break;
                  case 't': fprintf(fp,"t"); break;
                  case 'n': fprintf(fp,"n"); break;
                  default: break;
               }
            }
         }
         else {
            for (j=site[i].pos; j<min(seqLen[site[i].seq],site[i].pos+pwmLen); j++) {
               switch(rseq[site[i].seq][j]) {
                  case 'a': fprintf(fp,"a"); break;
                  case 'c': fprintf(fp,"c"); break;
                  case 'g': fprintf(fp,"g"); break;
                  case 'n': fprintf(fp,"n"); break;
                  default: break;
               }
            }
         }
         if (site[i].pos+pwmLen-seqLen[site[i].seq]>0) {
            for (j=seqLen[site[i].seq]; j<site[i].pos+pwmLen; j++) fprintf(fp,"x"); 
         }
         fprintf(fp,"\n");
      }
   }
   fclose(fp);
   if (fileName) { free(fileName); fileName=NULL; }
}

void print_result_2(Sites *site,int nsites,int numSeq,char **seq,char **rseq,double logev,double **opwm,int pwmLen,
   int id,char *sdyad,char *pwmConsensus,int *seqLen,int numCycle,double pvalueCutoff,FILE *fq,FILE *fpwm) {

   register int i,j;
   int cn[4];
   int *seqCn;

   seqCn=alloc_int(numSeq);

   for (i=0; i<numSeq; i++) seqCn[i]=0;
   for (i=0; i<nsites; i++) seqCn[site[i].seq]++; 
  
   for (i=0; i<4; i++) cn[i]=0; 
   for (i=0; i<numSeq; i++) {
      if (seqCn[i]==0) cn[0]++; 
      if (seqCn[i]==1) cn[1]++; 
      if (seqCn[i]==2) cn[2]++; 
      if (seqCn[i]>2)  cn[3]++; 
   }
   if (seqCn) { free(seqCn); seqCn=NULL; }

   fprintf(fq,"Cycle[%3d] motif[%d]:\n",numCycle,id);
   fprintf(fq,"   spaced dyad:\t\t\t%s\n",sdyad);
   fprintf(fq,"   motif consensus:\t\t%s\n",pwmConsensus);
   fprintf(fq,"\t\t\t\tm=ac r=ag w=at s=cg y=ct k=gt b=cgt d=agt h=act v=acg\n");
   fprintf(fq,"   motif length(w):\t\t%d\n",pwmLen);
   fprintf(fq,"   number of sites:\t\t%d\n",nsites);
   fprintf(fq,"   log(E-value):\t\t%9.2f\n",logev);
   fprintf(fq,"   pwm p-value cutoff:\t\t%e\n\n",pvalueCutoff);
   fprintf(fq,"   #seq_0_site  #seq_1_site  #seq_2_site  #seq_>2_site\n");
   fprintf(fq,"   %9d%10d%10d%10d\n",cn[0],cn[1],cn[2],cn[3]);
   fprintf(fq,"\nIndividual sites: [%2dbp flanking--MOTIF--%2dbp flanking] [strand] [seq] [pos] [p-value]\n",FLANKING_BASES,FLANKING_BASES);
   for (i=0; i<nsites; i++) {
      if (site[i].rev=='0') {
         if (site[i].pos<FLANKING_BASES) {
            if (site[i].pos<0) {
               for (j=0; j<FLANKING_BASES; j++) fprintf(fq," "); 
            }
            else {
               for (j=0; j<FLANKING_BASES-site[i].pos; j++) fprintf(fq," "); 
               for (j=0; j<site[i].pos; j++) fprintf(fq,"%c",seq[site[i].seq][j]);
            }
         }
         else {
            for (j=site[i].pos-FLANKING_BASES; j<site[i].pos; j++) fprintf(fq,"%c",seq[site[i].seq][j]);
         }

         if (site[i].pos<0) {
            for (j=site[i].pos; j<0; j++) fprintf(fq,"X"); 
            for (j=0; j<pwmLen+site[i].pos; j++) {
               switch(seq[site[i].seq][j]) {
                  case 'a': fprintf(fq,"A"); break;
                  case 'c': fprintf(fq,"C"); break;
                  case 'g': fprintf(fq,"G"); break;
                  case 't': fprintf(fq,"T"); break;
                  case 'n': fprintf(fq,"N"); break;
                  default: break;
               }
            }
         }
         else {
            for (j=site[i].pos; j<min(seqLen[site[i].seq],site[i].pos+pwmLen); j++) {
               switch(seq[site[i].seq][j]) {
                  case 'a': fprintf(fq,"A"); break;
                  case 'c': fprintf(fq,"C"); break;
                  case 'g': fprintf(fq,"G"); break;
                  case 't': fprintf(fq,"T"); break;
                  case 'n': fprintf(fq,"N"); break;
                  default: break;
               }
            }
         }
         if (site[i].pos+pwmLen-seqLen[site[i].seq]>0) {
            for (j=seqLen[site[i].seq]; j<site[i].pos+pwmLen; j++) fprintf(fq,"X"); 
         }
         // print flanking region
         for (j=site[i].pos+pwmLen; j<min(site[i].pos+pwmLen+FLANKING_BASES,seqLen[site[i].seq]); j++) 
            fprintf(fq,"%c",seq[site[i].seq][j]);
         if (site[i].pos+pwmLen+FLANKING_BASES>seqLen[site[i].seq]) { 
            for (j=0; j<site[i].pos+pwmLen+FLANKING_BASES-seqLen[site[i].seq]; j++) fprintf(fq," "); 
         }
         fprintf(fq,"\t+\t%d\t%d\t%e\n",site[i].seq+1,site[i].pos+1,site[i].pvalue);
      }
      else {
         if (site[i].pos<FLANKING_BASES) {
            if (site[i].pos<0) {
               for (j=0; j<FLANKING_BASES; j++) fprintf(fq," "); 
            }
            else {
               for (j=0; j<FLANKING_BASES-site[i].pos; j++) fprintf(fq," "); 
               for (j=0; j<site[i].pos; j++) fprintf(fq,"%c",rseq[site[i].seq][j]);
            }
         }
         else {
            for (j=site[i].pos-FLANKING_BASES; j<site[i].pos; j++) fprintf(fq,"%c",rseq[site[i].seq][j]);
         }

         if (site[i].pos<0) {
            for (j=site[i].pos; j<0; j++) fprintf(fq,"X"); 
            for (j=0; j<pwmLen+site[i].pos; j++) {
               switch(rseq[site[i].seq][j]) {
                  case 'a': fprintf(fq,"A"); break;
                  case 'c': fprintf(fq,"C"); break;
                  case 'g': fprintf(fq,"G"); break;
                  case 't': fprintf(fq,"T"); break;
                  case 'n': fprintf(fq,"N"); break;
                  default: break;
               }
            }
         }
         else {
            for (j=site[i].pos; j<min(seqLen[site[i].seq],site[i].pos+pwmLen); j++) {
               switch(rseq[site[i].seq][j]) {
                  case 'a': fprintf(fq,"A"); break;
                  case 'c': fprintf(fq,"C"); break;
                  case 'g': fprintf(fq,"G"); break;
                  case 't': fprintf(fq,"T"); break;
                  case 'n': fprintf(fq,"N"); break;
                  default: break;
               }
            }
         }
         if (site[i].pos+pwmLen-seqLen[site[i].seq]>0) {
            for (j=seqLen[site[i].seq]; j<site[i].pos+pwmLen; j++) fprintf(fq,"X"); 
         }
         // print flanking region
         for (j=site[i].pos+pwmLen; j<min(site[i].pos+pwmLen+FLANKING_BASES,seqLen[site[i].seq]); j++) 
            fprintf(fq,"%c",rseq[site[i].seq][j]);
         if (site[i].pos+pwmLen+FLANKING_BASES>seqLen[site[i].seq]) { 
            for (j=0; j<site[i].pos+pwmLen+FLANKING_BASES-seqLen[site[i].seq]; j++) fprintf(fq," "); 
         }
         fprintf(fq,"\t-\t%d\t%d\t%e\n",site[i].seq+1,seqLen[site[i].seq]-site[i].pos,site[i].pvalue);
      }
   }
   fprintf(fq,"\n");
   fprintf(fpwm,">m%d_%s\n",id,pwmConsensus);
   for (i=0; i<4; i++) {
      for (j=0; j<pwmLen; j++) {
         if (j<pwmLen-1) fprintf(fpwm,"%5.3f\t",opwm[j][i]);
         else fprintf(fpwm,"%5.3f\n",opwm[j][i]);
      }
   }
   fprintf(fq,"-------------------------------------------------------\n");
   fflush(fq);
   fflush(fpwm);
}

